Package: Ternary
Version: 2.3.3
Title: Create Ternary and Holdridge Plots
Description: Plots ternary diagrams (simplex plots / Gibbs triangles) and
  Holdridge life zone plots <doi:10.1126/science.105.2727.367> using the
  standard graphics functions.
  Allows custom annotation, interpolating, contouring and scaling of plotting
  region.
  Includes a 'Shiny' user interface for point-and-click ternary plotting.
  An alternative to 'ggtern', which uses the 'ggplot2' family of plotting 
  functions.
URL: https://ms609.github.io/Ternary/,
        https://github.com/ms609/Ternary/
BugReports: https://github.com/ms609/Ternary/issues/
Authors@R: c(person("Martin R.", "Smith",
                    email = "martin.smith@durham.ac.uk",
                    role = c("aut", "cre", "cph"), 
                    comment = c(ORCID = "0000-0001-5660-1727")
                   ),
             person("Lilian", "Sanselme", role = "ctb")
            )
License: GPL (>= 2)
Language: en-GB
Depends: R (>= 3.6.0)
Imports: PlotTools (>= 0.2.0), RcppHungarian, shiny, sp,
Suggests: colourpicker, knitr, readxl, rmarkdown, shinyjs, spelling,
        testthat (>= 3.0), vdiffr,
Config/Needs/check: rcmdcheck
Config/Needs/coverage: covr
Config/Needs/memcheck: devtools, rcmdcheck
Config/Needs/metadata: codemeta
Config/Needs/revdeps: revdepcheck
Config/Needs/website: pkgdown
Config/testthat/edition: 3
Config/testthat/parallel: false
LazyData: true
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
Author: Martin R. Smith [aut, cre, cph]
    (<https://orcid.org/0000-0001-5660-1727>),
  Lilian Sanselme [ctb]
Maintainer: Martin R. Smith <martin.smith@durham.ac.uk>
Built: R 4.4.1; ; 2024-08-30 16:53:05 UTC; unix
